import { useState } from "react";
import { motion, AnimatePresence } from "motion/react";
import { useTranslation } from "react-i18next";
import { useToast } from "../hooks/useToast";
import { useLoading } from "../hooks/useLoading";
import FormField from "../components/FormField";
import FormInput from "../components/FormInput";
import FormSelect from "../components/FormSelect";
import Button from "../components/Button";
import { lockerService } from "../services/locker.service";
import clsx from "clsx";
import FormBox from "../components/FormBox";
import { Copy } from "lucide-react";

function KeySwap() {
  const { t } = useTranslation();
  const showToast = useToast();
  const { isLoading, setIsLoading } = useLoading();
  const [notification, setNotification] = useState({
    visible: false,
    message: "",
    type: "",
  });
  const [keySwapDetails, setKeySwapDetails] = useState({
    cabinetId: "",
    lockerId: "",
    reason: "",
    oldKey: "",
    newKey: "",
    newKeyConfirm: "",
    cabinetIdValid: true,
    lockerIdValid: true,
    reasonValid: true,
    oldKeyValid: true,
    newKeyValid: true,
    newKeyConfirmValid: true,
  });

  const formFields = [
    {
      name: "cabinetId",
      label: t("cabinetId"),
      type: "input",
      maxLength: 6,
      readOnly: isLoading,
      validate: (value) => /^[A-Z]{2}[0-9]{4}/.test(value),
    },
    {
      name: "lockerId",
      label: t("lockerId"),
      type: "input",
      maxLength: 6,
      readOnly: isLoading,
      validate: (value) => /^[A-Z]{2}[0-9]{4}/.test(value),
    },
    {
      name: "reason",
      label: t("reasonForChange"),
      type: "input",
      maxLength: 50,
      readOnly: isLoading,
      validate: (value) => value !== "",
    },
    {
      name: "oldKey",
      label: t("oldKeyId"),
      type: "input",
      maxLength: 6,
      readOnly: isLoading,
      validate: (value) => /^[A-Z]{2}[0-9]{4}/.test(value),
    },
    {
      name: "newKey",
      label: t("newKeyId"),
      type: "input",
      maxLength: 6,
      readOnly: isLoading,
      validate: (value) => /^[A-Z]{2}[0-9]{4}/.test(value),
    },
    {
      name: "newKeyConfirm",
      label: t("confirmNewKeyId"),
      type: "input",
      maxLength: 6,
      readOnly: isLoading,
      validate: (value) => value !== "" && value === keySwapDetails.newKey,
    },
  ];

  const handleKeySwap = async (e) => {
    e.preventDefault();
    let valid = true;

    for (const field of formFields) {
      if (!field.validate(keySwapDetails[field.name])) {
        valid = false;
        setKeySwapDetails((prev) => ({
          ...prev,
          [`${field.name}Valid`]: false,
        }));
      }
    }

    if (!valid) {
      showToast(t("highlightedFieldsInvalid"), "error");
      return;
    }

    setIsLoading(true);
    try {
      const response = await lockerService.keySwap(
        keySwapDetails.cabinetId,
        keySwapDetails.lockerId,
        keySwapDetails.reason,
        keySwapDetails.oldKey,
        keySwapDetails.newKey
      );
      if (response.status === 200) {
        setNotification({
          visible: true,
          message: response.data.message,
          type: "success",
        });
      } else {
        setNotification({
          visible: true,
          message: response.data.message,
          type: "error",
        });
      }
    } catch (error) {
      setNotification({
        visible: true,
        message: error.message,
        type: "error",
      });
    } finally {
      setIsLoading(false);
    }
  };

  const renderField = (field) => {
    const commonProps = {
      value: keySwapDetails[field.name],
      onChange: (e) => {
        const newLockerDetails = { ...keySwapDetails };
        newLockerDetails[field.name] = e.target.value.toUpperCase();
        newLockerDetails[`${field.name}Valid`] = true;
        setKeySwapDetails(newLockerDetails);
      },
      maxLength: field.maxLength,
      className: clsx(!keySwapDetails[`${field.name}Valid`] && "border-error"),
    };

    return (
      <FormField key={field.name} label={field.label} icon={field.icon}>
        {field.type === "input" ? (
          <FormInput
            {...commonProps}
            type={field.subType}
            readOnly={field.readOnly}
          />
        ) : (
          <FormSelect {...commonProps} options={field.options} />
        )}
      </FormField>
    );
  };

  return (
    <div>
    <AnimatePresence>
    {notification.visible && (
          <motion.div
            initial={{ opacity: 0 }}
            animate={{ opacity: 1 }}
            exit={{ opacity: 0 }}
            className={clsx(
              "p-4 mb-8 font-body text-center text-xl rounded-2xl flex items-center justify-center gap-2",
              notification.type === "error"
                ? "bg-error-surface text-error"
                : "bg-success-surface text-success"
            )}
          >
            {notification.message.split(":").map((msg, index) => {
              return index === 1 ? (
                <span key={index} className="border-b border-dashed">
                  {msg}
                </span>
              ) : (
                <span key={index}>{msg}</span>
              );
            })}
            <Copy
              cursor={"pointer"}
              size={15}
              onClick={navigator.clipboard.writeText(
                notification.message.split(":")[1].trim()
              )}
            />
          </motion.div>
        )}
    </AnimatePresence>
    <div className="relative">
    {notification.type === "success" && (
        <div className="absolute inset-0 bg-[#fff]/50 z-10 rounded-3xl" />
      )}
    <FormBox title={t("lockerStatus")}>
      <div className="p-2 pt-7 flex flex-col gap-4">
        {formFields.map(renderField)}
      </div>
      <Button text={t("submit")} onClick={handleKeySwap} />
    </FormBox>
    </div>
  </div>
  )
}

export default KeySwap;
